/* UpTools v8.6
 *
 * Copyright (c) 2005-2020 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPROUTE2_INL
#define UPROUTE2_INL

// Inline methods for UpRoute2

inline
unsigned int UpRoute2::getMaskLen() const
{ return maskLen; }

inline
void UpRoute2::set(const UpAddress& baseAddr,unsigned int _maskLen,unsigned int _metric) {
	m.setMask(_maskLen,baseAddr.addrType); // works only for INET or INET6
	if( m.addrType!=AF_UNSPEC ) { ip=baseAddr; maskLen = _maskLen; } else { ip.clear(); maskLen = 0; }
	metric=_metric;
}

inline
void UpRoute2::set(in_addr baseAddr,in_addr mask,unsigned int _metric)
{ ip=baseAddr; m = mask; metric=_metric; maskLen = m.maskBitsLen(); }


inline
void UpRoute2::set(const in6_addr& baseAddr,const in6_addr& mask,unsigned int _metric)
{ ip=baseAddr; m = mask; metric=_metric; maskLen = m.maskBitsLen(); }

inline
void UpRoute2::set(const UpAddress& baseAddr,const UpAddress& mask,unsigned int _metric) {
	if( baseAddr.family()!=mask.family() or
		!(mask.family()==AF_INET or mask.family()==AF_INET6) )
	{ ip.clear(); m.clear(); maskLen = 0; }
 	else { ip=baseAddr; m=mask; maskLen = m.maskBitsLen(); }
	metric=_metric;
}

inline
void UpRoute2::set(const UpRoute2& r)
{ ip=r.ip; m=r.m; metric=r.metric; maskLen = r.maskLen; }

inline
void UpRoute2::set(const UpNet& n,unsigned int _metric)
{ ip=n.ip; m=n.m; metric=_metric; maskLen = m.maskBitsLen(); }

inline
void UpRoute2::remove(const UpAddress& _ip,const UpAddress& _mask,bool includeSubnets)
{ removeTest(_ip,_mask.maskBitsLen(),includeSubnets); }

inline
void UpRoute2::remove(const UpAddress& _ip,unsigned int _maskLen,bool includeSubnets)
{ removeTest(_ip,_maskLen,includeSubnets); }

inline
void UpRoute2::append(std::ostream& o) const
{ o<<ip<<'/'<<(unsigned int)maskLen<<' '<<metric; }

inline
void UpRoute2::append(std::string& o) const
{ o<<ip<<'/'<<(unsigned int)maskLen<<' '<<metric; }

inline
void UpRoute2::setMetric(unsigned int _metric)
{	metric = _metric; }

inline
std::ostream& operator<<(std::ostream& o,const UpRoute2& net)
{ net.append(o); return o; }

inline
std::string& operator<<(std::string& o,const UpRoute2& net)
{ net.append(o); return o; }

inline
const UpRoute2* UpRoute2::lookup(const UpSockAddr& sa,bool useMetric) const {
	switch( sa.family() ) {
		case AF_INET:  return lookup(sa.sa.in->sin_addr);
		case AF_INET6: return lookup(sa.sa.in6->sin6_addr);
		default: return 0;
	}
}

inline
const UpRoute2* UpRoute2::lookup(const UpAddress& ipAddr,bool useMetric) const {
	switch( ipAddr.family() ) {
		case AF_INET:  return lookup(ipAddr.upAddr.a4,useMetric);
		case AF_INET6: return lookup(ipAddr.upAddr.a6,useMetric);
		default: return 0;
	}
}

inline
const UpRoute2* UpRoute2::lookup(const UpNet& n,bool useMetric) const
{ return lookup(n.ip,n.getMaskLen(),useMetric); }

inline
UpRoute2& UpRoute2::operator=(const std::string& s)
{ set(s); return *this; }

inline
UpRoute2* UpRoute2::getNext() {
	UpRoute2* r((UpRoute2*)getNextPriv());
	while( r ) {
		if( !r->auxNet ) return r;
		r = (UpRoute2*)r->getNextPriv();
	}
	return r;
}

inline
const UpRoute2* UpRoute2::getNext() const {
	UpRoute2* r(((UpRoute2*)this)->getNextPriv());
	while( r ) {
		if( !r->auxNet ) return r;
		r = r->getNextPriv();
	}
	return r;
}

inline
UpRoute2* UpRoute2::getNextPriv() {
	if(links[0]) return links[0];
	if(links[1]) return links[1];
	return parent ? parent->getNextPriv(order) : 0;
}

inline
UpRoute2* UpRoute2::getNextPriv(unsigned char ord) {
	if( ord==0 ) {
		if( links[1] ) return links[1];
	}
	return parent ? parent->getNextPriv(order) : 0;
}


 #endif
